//
// Copyright (c) 2009, Markus Rickert
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
// * Neither the name of the Technische Universitaet Muenchen nor the names of
//   its contributors may be used to endorse or promote products derived from
//   this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

#ifndef _RL_MATH_QUATERNION_H_
#define _RL_MATH_QUATERNION_H_

#define EIGEN_MATRIXBASE_PLUGIN <rl/math/MatrixBaseAddons.h>

#include <cassert>
#include <Eigen/Geometry>

#include "Real.h"

namespace rl
{
	namespace math
	{
		typedef ::Eigen::Quaternion< Real > Quaternion;
		
		namespace quaternion
		{
			inline
			void
			fromUniform(const Real& rand0, const Real& rand1, const Real& rand2, Quaternion& q)
			{
				assert(rand0 >= 0.0f);
				assert(rand0 <= 1.0f);
				assert(rand1 >= 0.0f);
				assert(rand1 <= 1.0f);
				assert(rand2 >= 0.0f);
				assert(rand2 <= 1.0f);
				
				Real sigma1 = ::std::sqrt(1.0f - rand0);
				Real sigma2 = ::std::sqrt(rand0);
				Real theta1 = 2.0f * static_cast< Real >(M_PI) * rand1;
				Real theta2 = 2.0f * static_cast< Real >(M_PI) * rand2;
				
				q = Quaternion(
					::std::sin(theta1) * sigma1,
					::std::cos(theta1) * sigma1,
					::std::sin(theta2) * sigma2,
					::std::cos(theta2) * sigma2
				);
				
				q.normalize();
			}
		}
	}
}

#endif // _RL_MATH_QUATERNION_H_
