//
// Copyright (c) 2009, Markus Rickert
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
//
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright notice,
//   this list of conditions and the following disclaimer in the documentation
//   and/or other materials provided with the distribution.
// * Neither the name of the Technische Universitaet Muenchen nor the names of
//   its contributors may be used to endorse or promote products derived from
//   this software without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
// CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
// SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
// INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
// CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
// ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//

#ifndef _RL_MDL_JOINT_H_
#define _RL_MDL_JOINT_H_

#include <rl/math/Vector.h>

#include "Transform.h"

namespace rl
{
	namespace mdl
	{
		class Joint : public Transform
		{
		public:
			Joint(const ::std::size_t& dof);
			
			virtual ~Joint();
			
			void forwardAcceleration();
			
			void forwardDynamics1();
			
			void forwardDynamics2();
			
			void forwardDynamics3();
			
			void forwardVelocity();
			
			const ::rl::math::Vector& getAcceleration() const;
			
			::std::size_t getDof() const;
			
			const ::rl::math::Vector& getMaximum() const;
			
			const ::rl::math::Vector& getMinimum() const;
			
			const ::rl::math::Vector& getPosition() const;
			
			const ::rl::math::Vector& getTorque() const;
			
			const ::rl::math::Vector& getSpeed() const;
			
			const ::rl::math::Vector& getVelocity() const;
			
			void inverseForce();
			
			void inverseVelocity1();
			
			void inverseVelocity2();
			
			void inverseVelocity3();
			
			void setAcceleration(const ::rl::math::Vector& qdd);
			
			virtual void setPosition(const ::rl::math::Vector& q) = 0;
			
			void setTorque(const ::rl::math::Vector& tau);
			
			void setVelocity(const ::rl::math::Vector& qd);
			
			::rl::math::MotionVector a;
			
			::rl::math::MotionVector c;
			
			::rl::math::Matrix D;
			
			::rl::math::Vector max;
			
			::rl::math::Vector min;
			
			::rl::math::Vector offset;
			
			::rl::math::Vector q;
			
			::rl::math::Vector qd;
			
			::rl::math::Vector qdd;
			
			::rl::math::Matrix S;
			
			::rl::math::Vector speed;
			
			::rl::math::Vector tau;
			
			::rl::math::Vector u;
			
			::rl::math::Matrix U;
			
			::rl::math::MotionVector v;
			
			::Eigen::Matrix< bool, ::Eigen::Dynamic, 1 > wraparound;
			
		protected:
			
		private:
			
		};
	}
}

#endif // _RL_MDL_JOINT_H_
